use anyhow::{Result, bail};
use prelude::*;

pub mod app_common;
mod client;
mod install;
mod ip;
pub mod message;
mod prelude;
mod server;
pub mod tlv;

fn main() -> Result<()> {
    let rt = tokio::runtime::Builder::new_current_thread()
        .enable_io()
        .enable_time()
        .build()?;

    rt.block_on(async_main())?;

    Ok(())
}

async fn async_main() -> Result<()> {
    let mut args = env::args();

    let _exe_name = args.next();

    let Some(subcommand) = args.next() else {
        return Err(CliArgError::MissingSubcommand.into());
    };

    match subcommand.as_ref() {
        "--version" => {
            println!("lookaround v{}", env!("CARGO_PKG_VERSION"));
        }
        "client" => client::client(args).await?,
        "config" => {
            config();
        }
        "find-nick" => client::find_nick(args).await?,
        "install" => {
            let nickname = args.next();
            install::main(nickname.as_deref())?
        }
        "my-ips" => my_ips()?,
        "server" => server::server(args).await?,
        x => bail!("Unknown subcommand `{x}`"),
    }
    Ok(())
}

fn config() {
    if let Some(dir) = app_common::try_config_dir() {
        println!("config dir = `{dir:?}`");
    } else {
        println!("Can't detect config dir.");
    }
}

fn my_ips() -> Result<(), AppError> {
    for addr in ip::get_ips()? {
        println!("{:?}", addr);
    }

    Ok(())
}
